//
// Helper functions for TRT N-Tuple analysis framework - all those relating to hit-level objects
//

unsigned int indet_hit_vector_size(TRT *Trk) { // HLP
 unsigned int n = Trk->InDetTrackParticles_hit_gasType->size();
 return n;
}

unsigned int gsf_hit_vector_size(TRT *Trk) { // HLP
  unsigned int n = Trk->electron_gsfTrack_hit_gasType->size();
  return n;
}

bool indet_isHT(TRT *Trk, unsigned int ihit) { // HLP
  unsigned int word = Trk->InDetTrackParticles_hit_bitPattern->at(ihit);
  return (word & 0x00020000u);
}

bool gsf_isHT(TRT *Trk, unsigned int ihit) { // HLP
  unsigned int word = Trk->electron_gsfTrack_hit_bitPattern->at(ihit);
  return (word & 0x00020000u);
}

bool is_any_ht_bit_set(unsigned int word) { // HLP
  return (word & 0x04020100u);
}

bool is_middle_ht_bit_set(unsigned int word) { // HLP
  return (word & 0x00020000u);
}

void set_ht_bit(unsigned int &word) { // HLP
  word = word | 0x00020000u;
}

void print_bit_pattern(unsigned int word) { // HLP
/*
  Print out the bit pattern.
  In (recent) MC the following bits are zeroed:
  msb, and first 4 unused bits, first and third HT bits, last 4 LT bits
  The msb is set to one if it is MC overlayed.

   bit31                          bit0
   |    leading          trailing |
   #----HLLLLLLLLHLLLLLLLLHLLLLLLLL
   00000011111111111111111011110000
*/
  cout << "{";
  for (unsigned int k=0; k<32; k++) {
    unsigned int i = word & 0x80000000u;
    bool b = (i==0x80000000u);
    if (!((k-5)%9)) std::cout << " ";
    cout << b;
    if (!((k-5)%9)) std::cout << " ";
    word=word<<1;
  }
  cout << "}"<< endl;
}

unsigned int get_bit_pattern_bit(unsigned int word, unsigned int k) { // HLP
  word=word<<k;
  unsigned int i = word & 0x80000000u;
  bool b = (i==0x80000000u);
  return b;
}

/*

  A collection of helpers related to hit types and hit counts on a track.
  Boolean definitions of hit types (precision, tube, outlier) are such that on
  a track #isPrecise + #isTubehit + #isOutlier is equal to the hit vector size.
  This requires isPrecise to exclude precision hits that are also outliers.

  New: since _hit_type might not be available, hits types are now defined
       based on track-to-wire distance and hit error.

    t2w = Trk->InDetTrackParticles_hit_rTrackToWire->at(ihit);
    err = Trk->InDetTrackParticles_hit_error->at(ihit);
    isPre = t2w>0.0 && t2w<2.1 && err<1.0;
    isTub = t2w>0.0 && t2w<2.1 && err>=1.0;
    isOut = t2w==0.0 || t2w>=2.1;

    See https://indico.cern.ch/event/1255556/contributions/5274231/attachments/2599056/4487291/ntuple_hitdefinition_trtsw.pdf

  Notes:

    * Trk->InDetTrackParticles_nTRTHits is the number of non-outlier hits.
    * about 1% of precision hits are also outliers; so only use
      Trk->InDetTrackParticles_nTRTHitsPrecision if you intend to include precise outliers
    * The sum of isOutlier hits agrees with Trk->InDetTrackParticles_nTRTOutliers;

*/

// TODO: these will be defined as variable(s) in the xTRTTools in the future
//       In this case remove the functions (after verifying).

bool indet_isOutlier(TRT *Trk, unsigned int ihit) { // HLP
  double t2w = Trk->InDetTrackParticles_hit_rTrackToWire->at(ihit);
  bool isOut = t2w==0.0 || t2w>=2.1;
  return isOut;
}
bool indet_isTubehit(TRT *Trk, unsigned int ihit) { // HLP
  double t2w = Trk->InDetTrackParticles_hit_rTrackToWire->at(ihit);
  double err = Trk->InDetTrackParticles_hit_error->at(ihit);
  bool isTub = t2w>0.0 && t2w<2.1 && err>=1.0;
  return isTub;
}
bool indet_isPrecise(TRT *Trk, unsigned int ihit) { // HLP
  // not including precision hits that are also outliers
  double t2w = Trk->InDetTrackParticles_hit_rTrackToWire->at(ihit);
  double err = Trk->InDetTrackParticles_hit_error->at(ihit);
  bool isPre = t2w>0.0 && t2w<2.1 && err<1.0;
  return isPre;
}

// FIXME: t2w is not set for GSF tracks!
bool gsf_isOutlier(TRT *Trk, unsigned int ihit) { // HLP
  double t2w = Trk->electron_gsfTrack_hit_rTrackToWire->at(ihit);
  bool isOut = t2w==0.0 || t2w>=2.1;
  return isOut;
}
// FIXME: t2w is not set for GSF tracks!
bool gsf_isTubehit(TRT *Trk, unsigned int ihit) { // HLP
  double t2w = Trk->electron_gsfTrack_hit_rTrackToWire->at(ihit);
  double err = Trk->electron_gsfTrack_hit_error->at(ihit);
  bool isTub = t2w>0.0 && t2w<2.1 && err>=1.0;
  return isTub;
}
// FIXME: t2w is not set for GSF tracks!
bool gsf_isPrecise(TRT *Trk, unsigned int ihit) { // HLP
  double t2w = Trk->electron_gsfTrack_hit_rTrackToWire->at(ihit);
  double err = Trk->electron_gsfTrack_hit_error->at(ihit);
  // not including precision hits that are also outliers
  bool isPre = t2w>0.0 && t2w<2.1 && err<1.0;
  return isPre;
}

unsigned int indet_nOutlier(TRT *Trk) { // HLP
  unsigned int n=0;
  unsigned int hitvectorsize = indet_hit_vector_size(Trk);
  for (unsigned int ihit=0; ihit<hitvectorsize; ihit++) {
    if ( indet_isOutlier(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int indet_nTubehit(TRT *Trk) { // HLP
  unsigned int n=0;
  unsigned int hitvectorsize = indet_hit_vector_size(Trk);
  for (unsigned int ihit=0; ihit<hitvectorsize; ihit++) {
    if ( indet_isTubehit(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int indet_nPrecise(TRT *Trk) { // HLP
  unsigned int n=0;
  unsigned int hitvectorsize = indet_hit_vector_size(Trk);
  for (unsigned int ihit=0; ihit<hitvectorsize; ihit++) {
    if ( indet_isPrecise(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int gsf_nOutlier(TRT *Trk) { // HLP
  unsigned int n=0;
  unsigned int hitvectorsize = gsf_hit_vector_size(Trk);
  for (unsigned int ihit=0; ihit<hitvectorsize; ihit++) {
    if ( gsf_isOutlier(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int gsf_nTubehit(TRT *Trk) { // HLP
  unsigned int n=0;
  unsigned int hitvectorsize = gsf_hit_vector_size(Trk);
  for (unsigned int ihit=0; ihit<hitvectorsize; ihit++) {
    if ( gsf_isTubehit(Trk, ihit) ) n++;
  }
  return n;
}
unsigned int gsf_nPrecise(TRT *Trk) { // HLP
  unsigned int n=0;
  unsigned int hitvectorsize = gsf_hit_vector_size(Trk);
  for (unsigned int ihit=0; ihit<hitvectorsize; ihit++) {
    if ( gsf_isPrecise(Trk, ihit) ) n++;
  }
  return n;
}

/////////////////////////////////////////////
// Suite of "straw layer number" functions //
/////////////////////////////////////////////
/*
    _hit_bec        - Barrel-Endcap {-2,-1,1,2}
    _hit_layer      - module layer in bec: Barrel [3], Endcap [14]
    _hit_strawLayer - Straw layer in module layer: Barrel [19, 24, 30], Endcap [6x16, 8x8]
*/

unsigned int straw_layer_number(int bec, int layer, int straw) { // HLP
  /*
    Return the proper straw layer number of TRT *Trk using _hit_bec, _hit_later, _hit_strawLayer.
    That is, 0-72 for the barrel, and 0-159 for the endcap.
    This assumes you are paying attention to which _hit_bec region of the detector you are referring to.
  */
  unsigned int sln=straw;
  if ( abs(bec)==1 ) {
    if ( layer>0 ) sln += 19;
    if ( layer>1 ) sln += 24;
  } else if ( abs(bec)==2 ) {
    if ( layer<6 )  sln += layer*16;
    if ( layer>=6 ) sln += 6*16 + (layer-6)*8;
  }
  else WARN("Bad bec value in straw_layer_number().");
  if ( abs(bec)==1 && sln>72 )  WARN("Out of range straw_layer_number for the barrel in straw_layer_number().");
  if ( abs(bec)==2 && sln>159 ) WARN("Out of range straw_layer_number for the end-cap in straw_layer_number().");
  return sln;
}

unsigned int indet_straw_layer_number(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  int straw = Trk->InDetTrackParticles_hit_strawLayer->at(ihit);
  return straw_layer_number(bec, layer, straw);
}

unsigned int gsf_straw_layer_number(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  int straw = Trk->electron_gsfTrack_hit_strawLayer->at(ihit);
  return straw_layer_number(bec, layer, straw);
}

//////////////////////////////////////////////////////
// End of - Suite of "straw layer number" functions //
//////////////////////////////////////////////////////

/////////////////////////////////////////
// Suite of "region boolean" functions //
/////////////////////////////////////////

bool isSideA(int bec) { // HLP
  if (bec>0) return true;
  if (bec<0) return false;
  WARN("Bad bec value in isSideA()", bec);
  return false;
}
bool isSideC(int bec) { // HLP
  if (bec<0) return true;
  if (bec>0) return false;
  WARN("Bad bec value in isSideC()", bec);
  return false;
}

bool isBarrel(int bec) { // HLP
  if (abs(bec)==1) return true;
  if (abs(bec)==2) return false;
  WARN("Bad bec value in isBarrel()", bec);
  return false;
}
bool isEndCap(int bec) { // HLP
  if (abs(bec)==2) return true;
  if (abs(bec)==1) return false;
  WARN("Bad bec value in isEndcap()", bec);
  return false;
}

bool isBarrelLayer0(int bec, int layer) { // HLP
  if (isEndCap(bec)) return false;
  if (layer<0 || layer>2) WARN("Bad bec value in isBarrelLayer0()", layer);
  if (layer==0) return true;
  return false;
}
bool isBarrelLayer1(int bec, int layer) { // HLP
  if (isEndCap(bec)) return false;
  if (layer<0 || layer>2) WARN("Bad bec value in isBarrelLayer1()", layer);
  if (layer==1) return true;
  return false;
}
bool isBarrelLayer2(int bec, int layer) { // HLP
  if (isEndCap(bec)) return false;
  if (layer<0 || layer>2) WARN("Bad bec value in isBarrelLayer2()", layer);
  if (layer==2) return true;
  return false;
}

bool isEndCapWheelA(int bec, int layer) { // HLP
  if (isBarrel(bec)) return false;
  if (layer<0 || layer>13) WARN("Bad bec value in isEndcapWheelA()", layer);
  if (layer<6) return true;
  return false;
}
bool isEndCapWheelB(int bec, int layer) { // HLP
  if (isBarrel(bec)) return false;
  if (layer<0 || layer>13) WARN("Bad bec value in isEndcapWheelA()", layer);
  if (layer>5) return true;
  return false;
}

// InDetTracks ...

// Sides
bool indet_isSideA(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isSideA(bec);
}
bool indet_isSideC(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isSideC(bec);
}

// Barrel
bool indet_isBarrel(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isBarrel(bec);
}
bool indet_isBarrelLayer0(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer0(bec, layer);
}
bool indet_isBarrelLayer1(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer1(bec, layer);
}
bool indet_isBarrelLayer2(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isBarrelLayer2(bec, layer);
}

// End-cap
bool indet_isEndCap(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  return isEndCap(bec);
}
bool indet_isEndCapWheelA(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isEndCapWheelA(bec, layer);
}
bool indet_isEndCapWheelB(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  return isEndCapWheelB(bec, layer);
}

// GSFTracks ...

// Sides
bool gsf_isSideA(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  return isSideA(bec);
}
bool gsf_isSideC(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  return isSideC(bec);
}

// Barrel
bool gsf_isBarrel(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  return isBarrel(bec);
}
bool gsf_isBarrelLayer0(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  return isBarrelLayer0(bec, layer);
}
bool gsf_isBarrelLayer1(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  return isBarrelLayer1(bec, layer);
}
bool gsf_isBarrelLayer2(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  return isBarrelLayer2(bec, layer);
}

// End-cap
bool gsf_isEndCap(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  return isEndCap(bec);
}
bool gsf_isEndCapWheelA(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  return isEndCapWheelA(bec, layer);
}
bool gsf_isEndCapWheelB(TRT *Trk, unsigned int ihit) { // HLP
  int   bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  return isEndCapWheelB(bec, layer);
}

///////////////////////////////////////////////
//End of Suite of "region boolean" functions //
///////////////////////////////////////////////

/*
  6 TRT partitions, for partition occupancy.
  Return values; positive for side A, negative for side C
  0:undetermined: 1:Barrel (3 layers): 2:Endcap wheel A (6 layers): 3:Endcap wheel B (8 layers)
*/

int indet_partition(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->InDetTrackParticles_hit_bec->at(ihit);
  if (abs(bec)==1) return bec; // Barrel A,C
  if (bec==0 || abs(bec)>2) {
    WARN("Bad bec value in indet_partition().");
    return 0;
  }
  int layer = Trk->InDetTrackParticles_hit_layer->at(ihit);
  if (layer<0) {
    WARN("Bad layer value in indet_partition().");
    return 0;
  }
  if (bec>0 && layer<6) return +2; // Endcap A Wheel A
  if (bec>0 && layer>5) return +3; // Endcap A Wheel B
  if (bec<0 && layer<6) return -2; // Endcap C Wheel A
  if (bec<0 && layer>5) return -3; // Endcap C wheel B
  WARN("Bad partition value in indet_partition().");
  return 0;
}

int gsf_partition(TRT *Trk, unsigned int ihit) { // HLP
  int bec = Trk->electron_gsfTrack_hit_bec->at(ihit);
  if (abs(bec)==1) return bec; // Barrel A,C
  if (bec==0 || abs(bec)>2) {
    WARN("Bad bec value in gsf_partition().");
    return 0;
  }
  int layer = Trk->electron_gsfTrack_hit_layer->at(ihit);
  if (layer<0) {
    WARN("Bad layer value in gsf_partition().");
    return 0;
  }
  if (bec>0 && layer<6) return +2; // Endcap A Wheel A
  if (bec>0 && layer>5) return +3; // Endcap A Wheel B
  if (bec<0 && layer<6) return -2; // Endcap C Wheel A
  if (bec<0 && layer>5) return -3; // Endcap C wheel B
  WARN("Bad partition value in gsf_partition().");
  return 0;
}
