//---------------------------------------------------------------------------------------------------
// PlotFits.C
//
// This root macro file gets residual histograms from "histograms.root" and fits them to
// gaussian functions to obtain mean and std of the distributions.
// The mean and std values are printed to a data file called "data.txt".
//
// Last update: Dec 2022
// Author     : Ahmet.Bingul@cern.ch
//---------------------------------------------------------------------------------------------------

#include "TrtBigPaperTool.cpp"
#include "../includes/TRT.C"
#include "../includes/helpers.cpp"


//input_root_file can be somting like "../histoRun2Zee_reweighted.root"

void PlotFits(string input_root_file) {

   gErrorIgnoreLevel = kWarning;

   int run = input_root_file.find("Run3");
   int sig = input_root_file.find("Zmm");
   int rwt = input_root_file.find("rewe");

   string png     = "png";
   string epoch   = run>0 ? "Run3"       : "Run2";
   string signal  = sig>0 ? "Zmm"        : "Zee";
   string reweight= rwt>0 ? "_reweighted": "_unweighted";
   string slash   = "/";
   string tnp     = "(T&P)";
   string decay   = sig ? "Z #rightarrow #mu^{+} #mu^{-}" : "Z #rightarrow e^{+} e^{-}";
   string caption = epoch + " " + decay + " " + tnp;
   string imgfolder = png + epoch + signal + reweight + slash;
   string datafile1 = "data" + epoch + signal + reweight + "1.txt";
   string datafile2 = "data" + epoch + signal + reweight + "2.txt";
   string datafile3 = "data" + epoch + signal + reweight + "3.txt";

   // input root file
   TFile *rootfile = new TFile(input_root_file.c_str());
   // the canvas
   TCanvas *c1 = new TCanvas("c1","c1",0,0,600,600);
   // output data file
   ofstream fout1(datafile1.c_str());
   ofstream fout2(datafile2.c_str());
   ofstream fout3(datafile3.c_str());

   fout1 << rootfile->GetName() << endl;
   fout2 << rootfile->GetName() << endl;

   TNamed *NTupleFile1 = (TNamed*) rootfile->Get("NTupleFile1");
   TNamed *NTupleFile2 = (TNamed*) rootfile->Get("NTupleFile2");
   fout3 << NTupleFile1->GetTitle() << endl;
   fout3 << NTupleFile2->GetTitle() << endl;

   // get list of histograms in root file ------------
   TIter   keyList(rootfile->GetListOfKeys());
   TKey    *key;
   Int_t   histono = 0;
   TH1D*   h1;

   for(int ip=0; ip<6; ip++){
   for(int ig=0; ig<2; ig++){
   for(int io=0; io<10;io++){
   for(int is=0; is<2; is++){

      string name = "hRes_" + to_string(ip) + to_string(ig) + to_string(io) + to_string(is);
      h1 = (TH1D *) rootfile->Get(name.c_str());
      int totalEntries = h1->GetEntries();

      if(h1==NULL){
         cout << "h1 = NULL pointer."<< endl;
         return;
      }

      // get mean and std of residuals via iterative gaussian function
      double mean=0, sigma=0, meanError=0, sigmaError=0;

      // print results to data files
      if(totalEntries<70){
         if(is==0) fout1 << h1->GetName() << "\t" << mean  << " " << meanError << "     "<< sigma << " " << sigmaError << endl;
         if(is==1) fout2 << h1->GetName() << "\t" << mean  << " " << meanError << "     "<< sigma << " " << sigmaError << endl;
      }
      else{
         iterGaus(h1, 1.5, 1.0e-4, 15, false, mean, meanError, sigma, sigmaError);
         if(is==0) fout1 << h1->GetName() << "\t" << mean  << " " << meanError << "     "<< sigma << " " << sigmaError << endl;
         if(is==1) fout2 << h1->GetName() << "\t" << mean  << " " << meanError << "     "<< sigma << " " << sigmaError << endl;
      }
      // generate png files
      string pngfile = imgfolder + name + ".png";
      c1->Print(pngfile.c_str());

   }}}}

   fout1.close();
   fout2.close();

   cout << "PlotFits.C ALL OK."<< endl;
}
