//---------------------------------------------------------------------------------------------------
// PlotResiduals.C
//
// This root macro file reads mean and std of residuals from data file generated by PlotFits.C.
// Plot residuals as function for Track Occupancy and detector, BA, BC, EAA, ... etc.
//
// Last update: Dec 2022
// Author     : Ahmet.Bingul@cern.ch
//---------------------------------------------------------------------------------------------------

#include "AtlasStyle.C"
#include "TrtBigPaperTool.cpp"
#include "../includes/TRT.C"
#include "../includes/helpers.cpp"
#include <string>

// -----------------------------------------------------------------------------------
void PlotResiduals(string input_root_file){

  gErrorIgnoreLevel = kWarning;

  int run = input_root_file.find("Run3");
  int sig = input_root_file.find("Zmm");
  int rwt = input_root_file.find("rewe");

  string png     = "png";
  string epoch   = run>0 ? "Run3"       : "Run2";
  string signal  = sig>0 ? "Zmm"        : "Zee";
  string reweight= rwt>0 ? "_reweighted": "_unweighted";
  string slash   = "/";
  string tnp     = "(T&P)";
  string decay   = sig>0 ? "Z #rightarrow #mu^{+} #mu^{-}" : "Z #rightarrow e^{+} e^{-}";
  string caption = epoch + " " + decay + " " + tnp;
  string imgfolder = png + epoch + signal + reweight + slash;
  string datafile1 = "data" + epoch + signal + reweight + "1.txt";
  string datafile2 = "data" + epoch + signal + reweight + "2.txt";

  gROOT->Reset();
  SetAtlasStyle();
  // we have 6 detectors and 2 gas types (6*2=12) configurations
  const int m = 12;
  // we have 10 track occupancy intervals
  const int n = 10;

  //TCanvas      *c1 = new TCanvas("c1","c1",0,0,800,600);
  TGraphErrors *gr1, *gr2;
  TH1D         *hist1, *hist2;

  string histofile, key1, key2;
  double occ[n]    = {0}, occError[n]   = {0};
  double mean1[n]  = {0}, meanError1[n] = {0};
  double mean2[n]  = {0}, meanError2[n] = {0};
  double sigma1[n] = {0}, sigmaError1[n]= {0};
  double sigma2[n] = {0}, sigmaError2[n]= {0};

  ifstream fin1(datafile1.c_str());
  ifstream fin2(datafile2.c_str());
  fin1 >> histofile;
  fin2 >> histofile;

  // Andrew's TrtBigPaperOptions
  TrtBigPaperToolOptions *h_opt = new TrtBigPaperToolOptions();
  h_opt->status    = "Internal";
  h_opt->sample0   = "Simulation";
  h_opt->sample1   = "Data";
  h_opt->x_title   = "TRT Track Occupancy";
  h_opt->y_title   = "Position Accuracy [mm]";
  h_opt->entriesbw = false;
  h_opt->statbox   = false;
  h_opt->ratio     = true;
  h_opt->showinfo  = false;
  h_opt->min       = 0.0;
  h_opt->max       = 0.2;
  h_opt->r_min     = 0.8;
  h_opt->r_max     = 1.2;
  h_opt->text[0] = {0.2,0.75,caption};

  // get the mean and std values from data file
   int xenon, argon, ba,bc,eaa,eab,eca,ecb;
   int ip, ig;
   string gas = "(Ar)", Text;

   for(int j=0; j<m; j++){
    for(int io=0; io<n; io++){

       fin1 >> key1;
       fin2 >> key2;
       if( fin1.eof() || fin2.eof() ) break;

       // file format
       //     pgos
       //hRes_xxxx
       string s1 = key1.substr(5,1);
       string s2 = key1.substr(6,1);
       ip = atoi(s1.c_str());
       ig = atoi(s2.c_str());

       fin1 >> mean1[io] >> meanError1[io] >> sigma1[io] >> sigmaError1[io];
       fin2 >> mean2[io] >> meanError2[io] >> sigma2[io] >> sigmaError2[io];
       if(sigmaError1[io]>0.2) sigmaError1[io]=0.2;
       if(sigmaError2[io]>0.2) sigmaError2[io]=0.2;
       occ[io] = 0.1/2 + io*0.1;
       occError[io] = 0.1/2;

    } // for io

    gr1 = new TGraphErrors(n,occ,sigma1,occError,sigmaError1);
    gr1->SetName(key1.c_str());

    gr2 = new TGraphErrors(n,occ,sigma2,occError,sigmaError2);
    gr2->SetName(key2.c_str());

    gas = (ig==0) ? "(Xe)" : "(Ar)";
    if(ip==0)  Text = "Barrel Side A "+gas;
    if(ip==1)  Text = "Barrel Side C "+gas;
    if(ip==2)  Text = "Endcap Side A Wheel A "+gas;
    if(ip==3)  Text = "Endcap Side A Wheel B "+gas;
    if(ip==4)  Text = "Endcap Side C Wheel A "+gas;
    if(ip==5)  Text = "Endcap Side C Wheel B "+gas;

    cout << ip <<" "<< ig << endl;

    h_opt->text[1] = {0.20,0.15,Text};

    hist1 = trt_big_paper_tool_wrapper(gr1);
    hist2 = trt_big_paper_tool_wrapper(gr2);

    string pngfile  = imgfolder + "ressigma" + to_string(10+j);
    h_opt->filename = pngfile;
    TrtBigPaperToolPlot(hist1, hist2, h_opt);

  } // for j

  fin1.close();
  fin2.close();

} // EOF
