#include <iostream>
#include <string>
#include <vector>
using namespace std;

#include "TStyle.h"
#include <TCanvas.h>
#include "TH1D.h"
#include "TLine.h"
#include "TLegend.h"
#include "TLatex.h"
#include "TH1F.h"
#include "TProfile.h"
#include "TGraphErrors.h"
#include "TEfficiency.h"

/*
  TH1D wrappers
  TrtBigPaperToolPlot() takes TH1D as arguments.
  The following provide wrappers to convert from
  TH1F, TProfile, TGraphErrors and TEfficiency
*/

TH1D* trt_big_paper_tool_wrapper(TH1F *h) {
  string w_name = "w_"+(string)h->GetName();
  TH1D *wrapped = (TH1D*)h->Clone(w_name.c_str());
  return wrapped;
}

TH1D* trt_big_paper_tool_wrapper(TProfile *p) {
  unsigned int nbins = p->GetNbinsX();
  double   bin_width = p->GetXaxis()->GetBinWidth(1);
  double xmin = p->GetXaxis()->GetBinLowEdge(1);
  double xmax = xmin + nbins*bin_width;
  string w_name = "w_"+(string)p->GetName();
  TH1D *wrapped = new TH1D(w_name.c_str(),w_name.c_str(),nbins,xmin,xmax);
  // TH1 index runs 0 to n+1 to include the over/underflows
  for (unsigned int i=0; i<=nbins+1; i++) {
    double y = p->GetBinContent(i);
    double e = p->GetBinError(i);
    wrapped->SetBinContent(i,y);
    wrapped->SetBinError(i,e);
  }
  return wrapped;
}

TH1D* trt_big_paper_tool_wrapper(TGraphErrors *g) {
  unsigned int nbins = g->GetN();
  double bin_width = g->GetX()[1] - g->GetX()[0];
  double xmin = TMath::MinElement(g->GetN(),g->GetX());
  double xmax = TMath::MaxElement(g->GetN(),g->GetX());
  string w_name = "w_"+(string)g->GetName();
  TH1D *wrapped = new TH1D(w_name.c_str(),w_name.c_str(),nbins,xmin-bin_width/2,xmax+bin_width/2);
  for (unsigned int i=0; i<nbins; i++) {
    // TGraph index runs 0 to n-1
    double y  = g->GetY()[i];
    double ey = g->GetErrorY(i);
    // TH1 index runs 1 to n
    wrapped->SetBinContent(i+1,y);
    wrapped->SetBinError(i+1,ey);
  }
  return wrapped;
}

TH1D* trt_big_paper_tool_wrapper(TEfficiency *te) {
  TH1* w_clone = te->GetCopyTotalHisto();
  unsigned int nbins = w_clone->GetNbinsX();
  double   bin_width = w_clone->GetXaxis()->GetBinWidth(1);
  double   xmin = w_clone->GetXaxis()->GetBinLowEdge(1);
  double   xmax = xmin + nbins*bin_width;
  delete w_clone;
  string w_name = "w_"+(string)te->GetName();
  TH1D *wrapped = new TH1D(w_name.c_str(),w_name.c_str(),nbins,xmin,xmax);
  // TH1 index runs 0 to n+1 to include the over/underflows
  for (unsigned int i=0; i<=nbins+1; i++) {
    double eff = te->GetEfficiency(i);
    double elo = te->GetEfficiencyErrorLow(i);
    double ehi = te->GetEfficiencyErrorUp(i);
    // TH1 cannot handle asymmetric errors.
    // Take the mean error, or very small if eff=0.
    // This is only problematic for efficencies close to 0 or 1:
    // pad1 error bars can be out of range, pad2 errors are underestimated.
    double err = 0.01;
    if (eff>0.) err=(elo+ehi)/2;
    wrapped->SetBinContent(i,eff);
    wrapped->SetBinError(i,err);
  }
  return wrapped;
}

/****** End Wrappers ******/

struct TrtBigPaperToolText{
  double x;
  double y;
  string s;
};

class TrtBigPaperToolOptions {

  public:

  string status, x_title, y_title, filename, sample0, sample1;
  double sample0_weight, sample1_weight, padfrac, min, max, r_min, r_max;
  bool   entriesbw, yellow, ratio, logy, one, errorbars, statbox, showinfo;
  int    normalize, outliers;
  TrtBigPaperToolText text[30];  // up to 10(user)+20(stat) lines of text {x,y,s}

  // Constructor (defaults)
  TrtBigPaperToolOptions() {

    status = "";
    x_title = "";
    y_title = "";
    filename = "";
    sample0="";
    sample1="";

    sample0_weight = 1.;
    sample1_weight = 1.;
    padfrac = 0.67;
    min = 0.;
    max = 0.;
    r_min = 0.;
    r_max = 0.;

    entriesbw = false;
    yellow = false;
    ratio = false;
    logy = false;
    errorbars = true;
    one = false;
    statbox = false;
    showinfo = false;

    normalize = 0;
    outliers = 0;

    for (int i=0; i<30; i++) {
      text[i].x = 0.;
      text[i].y = 0.;
      text[i].s = "";
    }

  }

  void Print() {
    cout << "--- Print TrtBigPaperTool Options ---" << endl;
    cout << "(string)status      - e.g '', 'Place holder', 'Internal', 'Preliminary', 'Final' --> '"                        << this->status      << "'" << endl;
    cout << "(string)x_title     - Override the original histogram x-axis title e.g. '', 'Position residual [#mum]' --> '"  << this->x_title     << "'" << endl;
    cout << "(string)y_title     - Override the original histogram y-axis title e.g. '', 'Position residual [#mum]' --> '"  << this->y_title     << "'" << endl;
    cout << "(string)filename    - Override the filename that is determined filename from the histogram --> '"              << this->filename    << "'" << endl;
    cout << "(string)sample0        - Override the default 'Sample 0' label for the legend and ratio title --> '"           << this->sample0        << "'" << endl;
    cout << "(string)sample1        - Override the default 'Sample 1' label for the legend and ratio title --> '"           << this->sample1        << "'" << endl;
    cout << "(double)sample0_weight  - sample0 event/track weight. Set to 1. if you don't want event weighting --> '"       << this->sample0_weight << "'" << endl;
    cout << "(double)sample1_weight  - sample1 event/track weight. Set to 1. if you don't want event weighting --> '"       << this->sample1_weight << "'" << endl;
    cout << "(double)padfrac     - Fraction that the plot takes up in pad 1. [0.1,0.9], typically two-thirds --> '"         << this->padfrac     << "'" << endl;
    cout << "(double)min         - Override the automatic calculation of the plot minimum --> '"                            << this->min         << "'" << endl;
    cout << "(double)max         - Override the automatic calculation of the plot maximum --> '"                            << this->max         << "'" << endl;
    cout << "(double)r_min       - Override the automatic calculation of the ratio plot minimum --> '"                      << this->r_min       << "'" << endl;
    cout << "(double)r_max       - Override the automatic calculation of the ratio plot maximum --> '"                      << this->r_max       << "'" << endl;
    cout << "  (bool)entriesbw   - Display the y-axis title in the form 'Entries / binwidth unit' --> '"                    << this->entriesbw   << "'" << endl;
    cout << "  (bool)yellow      - Yellow-fill sample0 histogram --> '"                                                     << this->yellow      << "'" << endl;
    cout << "  (bool)ratio       - Insert a ratio sub-plot --> '"                                                           << this->ratio       << "'" << endl;
    cout << "  (bool)logy        - log10 scale on the y-axis with minimum 1.0 --> '"                                        << this->logy        << "'" << endl;
    cout << "  (bool)one         - displaying only one histogram. Requires the call: TrtBigPaperToolPlot(h1, h1, h_opt); --> '" << this->one         << "'" << endl;
    cout << "  (bool)errorbars   - show error bars (default is true) --> '"                                                 << this->errorbars   << "'" << endl;
    cout << "  (bool)statbox     - displaying stat box (instead of the ATLAS label)                                  --> '" << this->statbox     << "'" << endl;
    cout << "  (bool)showinfo    - Show detailed information processing --> '"                                              << this->showinfo    << "'" << endl;
    cout << "   (int)normalize   - Normalization (0:none, 1:entries, 2:integral, 3:sample0/sample1 entries, 4:sample0/sample1 integral) --> '"  << this->normalize   << "'" << endl;
    cout << "   (int)outliers    - Number of most outlying ratio points to omit from the automatic ranger --> '"            << this->outliers    << "'" << endl;
    cout << "(TrtBigPaperToolText)text - Plot text in the form {(double) x, (double) y, (string) text}:" << endl;
    for (int i=0; i<30; i++) cout << "                    --> " << this->text[i].x << "', '" << this->text[i].y << "', '" << this->text[i].s  << "'" << endl;
    cout << "---------------------------------" << endl;
  }

};

void trt_big_paper_tool_ratio_range(
  TH1D *ratio_0_1,
  double &r_val_min, double &r_val_max,
  double &r_val_mean, double &r_err_mean,
  int h_outliers,
  double &fbins
) {

  // loop over (size h_outliers) omitting the most outlying value each time such that h_outliers
  // are omitted from the determination of r_val_min, r_val_max, r_val_mean, r_err_mean.

  unsigned int mbins=0;
  unsigned int nbins = ratio_0_1->GetNbinsX(); // number of histogram bins
  vector<bool> ok(nbins+1, true);
  double r_val, r_err, r_err2_sum;
  double diff, maxdiff;

  for (int j=0; j<h_outliers+1; j++) {

    // the zeroth iteration does not kill any outliers.
    r_val_min = ratio_0_1->GetBinContent(ratio_0_1->GetMaximumBin());
    r_val_max = ratio_0_1->GetBinContent(ratio_0_1->GetMinimumBin());

    mbins=0;
    r_err2_sum = 0.;
    r_val_mean = 0.;

    for (unsigned int i=1; i<=nbins; i++) {
      r_val = ratio_0_1->GetBinContent(i);
      r_err = ratio_0_1->GetBinError(i);
      if (r_err==0.) ok[i]=false;
      if (ok[i]) {
        mbins++;
        r_err2_sum += 1./(r_err*r_err);
        r_val_mean += r_val/(r_err*r_err);
        if (r_val<r_val_min) r_val_min=r_val;
        if (r_val>r_val_max) r_val_max=r_val;
      }
    } // i loop

    r_val_mean /= r_err2_sum; // weighted mean ratio
    if (mbins>0) r_err_mean = 1./sqrt(r_err2_sum/mbins); // weighted mean error
    else         r_err_mean = 0.;
    // loop over all contributing bins and flag false the mosr outlying one.

    diff=0.;
    maxdiff=0.;
    unsigned int k=0; // index of the most outlying
    for (unsigned int i=1; i<=nbins; i++) {
      r_val = ratio_0_1->GetBinContent(i);
      if (ok[i]) {
        diff = fabs(r_val_mean-r_val);
        if (diff>maxdiff) {
          maxdiff=diff;
          k=i;
        }
      }
    } // i loop
    ok[k]=false;

  } // j loop

  fbins = double(mbins)/nbins;
  //cout << count(ok.begin(), ok.end(), false) << endl;

} // trt_big_paper_tool_ratio_range

void TrtBigPaperToolPlot(
  TH1D  *h_0,         // sample0 histogram
  TH1D  *h_1,         // sample1 histogram
  TrtBigPaperToolOptions *h_opt // Options!
) {

  // unpack the options from TrtBigPaperToolOptions()
  string h_status      = h_opt->status;
  string h_filename    = h_opt->filename;
  string h_sample0        = h_opt->sample0;
  string h_sample1        = h_opt->sample1;
  string h_x_title     = h_opt->x_title;
  string h_y_title     = h_opt->y_title;
  double h_sample0_weight   = h_opt->sample0_weight;
  double h_sample1_weight   = h_opt->sample1_weight;
  double h_padfrac     = h_opt->padfrac;
  double h_min         = h_opt->min;
  double h_max         = h_opt->max;
  double h_r_min       = h_opt->r_min;
  double h_r_max       = h_opt->r_max;
  bool   h_showinfo    = h_opt->showinfo;
  bool   h_entriesbw   = h_opt->entriesbw;
  bool   h_yellow      = h_opt->yellow;
  bool   h_ratio       = h_opt->ratio;
  bool   h_logy        = h_opt->logy;
  bool   h_one         = h_opt->one;
  bool   h_errorbars   = h_opt->errorbars;
  bool   h_statbox     = h_opt->statbox;
  int    h_normalize   = h_opt->normalize;
  int    h_outliers    = h_opt->outliers;
  TrtBigPaperToolText text[30];
  for (int i=0; i<30; i++) text[i] = h_opt->text[i];

  // print the list of options
  //h_opt->Print();

  // Colors!
  const string CRed    = "\033[0;31m";
  const string CGreen  = "\033[0;32m";
  const string CYellow = "\033[0;33m";
  const string CBlue   = "\033[0;34m";
  const string CGrey   = "\033[0;90m";
  const string CNone   = "\033[0m";

  gStyle->SetLineWidth(1);
  gStyle->SetOptTitle(0);
  gStyle->SetOptStat(0);

  string h_0_name = h_0->GetName();
  string h_1_name = h_1->GetName();

  string canvas_name;
  if (h_filename=="") {
    canvas_name = "plot_"+h_0_name+"_"+h_1_name;
  } else {
    canvas_name = h_filename;
  }

  TCanvas *canvas = new TCanvas(canvas_name.c_str(), canvas_name.c_str(), 700, 700);
  //cout << CGreen << "  TrtBigPaperToolPlot: " << canvas_name << CNone << endl; //AB this line is commented out

  if (h_sample0=="") h_sample0=h_0_name;
  if (h_sample1=="") h_sample1=h_1_name;
  if (h_showinfo) cout << CBlue << "  TrtBigPaperToolPlot:  first histogram is named " << h_sample0 << CNone << endl;
  if (h_showinfo) cout << CBlue << "  TrtBigPaperToolPlot: second histogram is named " << h_sample1 << CNone << endl;

  //
  // Get histogram properties entries, integral, x-axis bins, bin width, range, span
  //
  double h_0_entries  = h_0->GetEntries();
  double h_1_entries  = h_1->GetEntries();
  double h_0_integral = h_0->Integral();
  double h_1_integral = h_1->Integral();
  double h_0_bin_width = h_0->GetXaxis()->GetBinWidth(1);
  double h_1_bin_width = h_1->GetXaxis()->GetBinWidth(1);
  unsigned int h_0_n_bins = h_0->GetNbinsX();
  unsigned int h_1_n_bins = h_1->GetNbinsX();
  double h_0_underflow = h_0->GetBinContent(0);
  double h_1_underflow = h_1->GetBinContent(0);
  double h_0_overflow = h_0->GetBinContent(h_0_n_bins+1);
  double h_1_overflow = h_1->GetBinContent(h_1_n_bins+1);
  double h_span = h_0_n_bins*h_0_bin_width;
  double h_xmin = h_0->GetXaxis()->GetBinLowEdge(1);
  double h_xmax = h_xmin+h_span;

  double h_0_mean = h_0->GetMean();
  double h_1_mean = h_1->GetMean();
  double h_0_std = h_0->GetStdDev();
  double h_1_std = h_1->GetStdDev();

  if (h_statbox) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: displaying stat box (instead o the ATLAS label)" << CNone << endl;
    string txt_mean_0      = "mean = " + to_string(h_0_mean);
    string txt_std_0       = " std = " + to_string(h_0_std);
    string txt_rms_0       = " rms = " + to_string(sqrt(h_0_std*h_0_std+h_0_mean*h_0_mean));
    string txt_entries_0   = "  entries = " + to_string((int)h_0_entries);
    string txt_underflow_0 = "underflow = " + to_string((int)h_0_underflow);
    string txt_overflow_0  = " overflow = " + to_string((int)h_0_overflow);
    string txt_integral_0  = " integral = " + to_string((int)h_0_integral);
    text[10] = {0.20,0.89,txt_mean_0};
    text[11] = {0.20,0.86,txt_std_0};
    text[12] = {0.20,0.83,txt_rms_0};
    text[13] = {0.20,0.80,txt_entries_0};
    text[14] = {0.20,0.77,txt_integral_0};
    text[15] = {0.20,0.74,txt_underflow_0};
    text[16] = {0.20,0.71,txt_overflow_0};
    if (!h_one) {
      string txt_mean_1      = "mean = " + to_string(h_1_mean);
      string txt_std_1       = " std = " + to_string(h_1_std);
      string txt_rms_1       = " rms = " + to_string(sqrt(h_1_std*h_1_std+h_1_mean*h_1_mean));
      string txt_entries_1   = "  entries = " + to_string((int)h_1_entries);
      string txt_underflow_1 = "underflow = " + to_string((int)h_1_underflow);
      string txt_overflow_1  = " overflow = " + to_string((int)h_1_overflow);
      string txt_integral_1  = " integral = " + to_string((int)h_1_integral);
      //text[20] = {0.42,0.89,txt_mean_1}; //AB these lines are commented out
      //text[21] = {0.42,0.86,txt_std_1};
      //text[22] = {0.42,0.83,txt_rms_1};
      //text[23] = {0.42,0.80,txt_entries_1};
      //text[24] = {0.42,0.77,txt_integral_1};
      //text[25] = {0.42,0.74,txt_underflow_1};
      //text[26] = {0.42,0.71,txt_overflow_1};
    }
  }

  if (h_0_n_bins!=h_1_n_bins) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: the two histograms do not have the same number of bins!" << CNone << endl;
  }

  if (h_0_bin_width!=h_1_bin_width) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: the two histograms do not have the same bin width!" << CNone << endl;
  }

  if (h_x_title=="") {
    h_x_title = h_0->GetXaxis()->GetTitle();
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: using histogram x-title" << CNone << endl;
  } else {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: overriding histogram x-title" << CNone << endl;
  }

  if (h_y_title=="") {
    h_y_title = h_0->GetYaxis()->GetTitle();
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: using histogram y-title" << CNone << endl;
  } else {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: overriding histogram y-title" << CNone << endl;
  }

  string h_x_units="";
  string h_y_units="";
  int pos1, pos2;
  string str1, str2, str3;

  pos1 = h_x_title.find("[");
  if (pos1>=0) {
    string str1 = h_x_title.substr(0,pos1-1);
    string str2 = h_x_title.substr(pos1);
    pos2 = str2.find("]");
    if (pos2>=0) {
      string str3 = h_x_title.substr(pos1+1,pos2-1);
      h_x_title = str1;
      h_x_units = str3;
    }
  }
  pos1 = h_y_title.find("[");
  if (pos1>=0) {
    string str1 = h_y_title.substr(0,pos1-1);
    string str2 = h_y_title.substr(pos1);
    pos2 = str2.find("]");
    if (pos2>=0) {
      string str3 = h_y_title.substr(pos1+1,pos2-1);
      h_y_title = str1;
      h_y_units = str3;
    }
  }

  //
  // Check the input parameters
  //

  if (h_normalize<0 || h_normalize>4) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: normalization choice ("<< h_normalize << ") is out of range." << endl;
    cout << "     The available choices are:" << endl;
    cout << "       0:don't normalise," << endl;
    cout << "       1:normalise both sample0 and sample1 by their histogram entries," << endl;
    cout << "       2:normalise both sample0 and sample1 by their histogram integrals," << endl;
    cout << "       3:normalise sample0 to sample1 by histogram entries," << endl;
    cout << "       4:normalise sample0 to sample1 by histogram integrals." << endl;
    cout << "     Setting h_normalize to zero !" << CNone << endl;
    h_normalize=0;
  }
  if (h_sample0_weight==0.) cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: sample 0 histogram weight is zero. You won't see this histogram." << CNone << endl;
  if (h_sample1_weight==0.) cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: sample 1 histogram weight is zero. You won't see this histogram." << CNone << endl;
  if (h_sample0_weight<0.) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: sample0 histogram weight is negative! resetting to 1" << CNone << endl;
    h_sample0_weight=1.;
  }
  if (h_sample1_weight<0.) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: sample1 histogram weight is negative! resetting to 1" << CNone << endl;
    h_sample1_weight=1.;
  }

  if (h_outliers<0 || h_outliers>(int)h_0_n_bins) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: h_outliers (" << h_outliers
         << ") is out of range for your histogram! setting to zero." << CNone << endl;
    h_outliers=0;
  }

  string s_bin_width = Form("%g",h_0_bin_width);
  string s_x_title, s_y_title;
  if (h_x_units=="") s_x_title = h_x_title;
  else               s_x_title = h_x_title + " [" + h_x_units + "]";
  if (h_y_units=="") s_y_title = h_y_title;
  else               s_y_title = h_y_title + " [" + h_y_units + "]";

  // But set the y-title to the form "Entries / binwidth units" if that is the case
  if (h_entriesbw) s_y_title = "Entries / " + s_bin_width + " " + h_x_units;
  if (h_0_bin_width==1) s_y_title = "Entries"; //AB this line is added

  if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: sample0 histogram integral/entries: " << h_0_integral << " " << h_0_entries << CNone << endl;
  if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: sample1 histogram integral/entries: " << h_1_integral << " " << h_1_entries << CNone << endl;
  if (h_showinfo) cout << CBlue  << "  TrtBigPaperToolPlot: x-title: " << s_x_title << CNone << endl;
  if (h_showinfo) cout << CBlue  << "  TrtBigPaperToolPlot: y-title: " << s_y_title << CNone << endl;


  //
  // Apply the event/track weight OR normalization w.r.t. entries or integral.
  //

  bool b_0_weighting = fabs(1.0-h_sample0_weight)>0.000001;
  bool b_1_weighting = fabs(1.0-h_sample1_weight)>0.000001;

  if ( (b_0_weighting || b_1_weighting) && h_normalize>0) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: you are attempting to weight AND normalize!" << endl;;
    cout << "    h_sample0_weight = " << h_sample0_weight << endl;
    cout << "    h_sample1_weight = " << h_sample1_weight << endl;
    cout << "    h_normalize = " << h_normalize << endl;
    cout << "    Please EITHER weight OR normalize." << endl;
    cout << "    Will IGNORE THE WEIGHTS this time." << CNone << endl;
    b_0_weighting = false;
    b_1_weighting = false;
  }

  if (b_0_weighting || b_1_weighting) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: applying sample0 weight: " << h_sample0_weight << CNone << endl;
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: applying sample1 weight: " << h_sample1_weight << CNone << endl;
    h_0->Scale(h_sample0_weight);
    h_1->Scale(h_sample1_weight);
  } else {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: no scaling applied" << CNone << endl;
  }

  if (h_normalize==1) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: normalization with histogram (all) entries [now Arbitrary units]" << CNone << endl;
    h_0->Scale(1./(h_0_bin_width*h_0_entries));
    h_1->Scale(1./(h_0_bin_width*h_1_entries));
    s_y_title = "Arbitrary units";
  } else if (h_normalize==2) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: normalization with histogram (window) integral [now Arbitrary units]" << CNone << endl;
    h_0->Scale(1./(h_0_bin_width*h_0_integral));
    h_1->Scale(1./(h_0_bin_width*h_1_integral));
    s_y_title = "Arbitrary units";
  } else if (h_normalize==3) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: normalize sample0 entries to equal 1 entries" << CNone << endl;
    h_0->Scale(h_1_entries/h_0_entries);
    h_1->Sumw2();
    //if (!h_yellow) h_0->Sumw2();
  } else if (h_normalize==4) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: normalize sample0 integral to equal sample1 integral" << CNone << endl;
    h_0->Scale(h_1_integral/h_0_integral);
    h_1->Sumw2();
    //if (!h_yellow) h_0->Sumw2();
  } else {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: no normalization applied" << CNone << endl;
    if (!b_0_weighting && !b_1_weighting) {
      h_1->Sumw2();
      if (!h_yellow) h_0->Sumw2();
    }
  }

  if (h_min==0.0) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: set histogram minimum to " << h_min << CNone << endl;
  } else {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: set histogram minimum to " << h_min << " (user-defined)" << CNone << endl;
  }
  if (h_max==0.0) {
    double h_0_maxval = h_0->GetBinContent(h_0->GetMaximumBin()); // the scaled value
    double h_1_maxval = h_1->GetBinContent(h_1->GetMaximumBin()); // the scaled value
    h_max = h_0_maxval>h_1_maxval ? h_0_maxval/h_padfrac : h_1_maxval/h_padfrac;
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: set histogram maximum to " << h_max << " (" << h_padfrac << ")" << CNone << endl;
  } else {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: set histogram maximum to " << h_max << " (user-defined)" << CNone << endl;
  }

  if (h_logy) {
    if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: y-axis is log scale." << CNone << endl;
    h_0->SetMaximum(h_max*10.);
    //h_0->SetMinimum(1.);
    //h_1->SetMinimum(1.);
  } else {
    h_0->SetMaximum(h_max);
    h_0->SetMinimum(h_min);
    h_1->SetMinimum(h_min);
  }

  // Build the ratio
  TH1D *ratio_0_1 = (TH1D *) h_0->Clone();
  ratio_0_1->Divide(h_1);

  // (h_outliers + the number of zero entries) will be omitted from the
  // determination of r_val_min, r_val_max, r_val_mean, r_err_mean
  double r_val_min, r_val_max, r_val_mean, r_err_mean, fbins;
  trt_big_paper_tool_ratio_range(ratio_0_1, r_val_min, r_val_max, r_val_mean, r_err_mean, h_outliers, fbins);

  // override if requested
  if (h_r_min!=0.) {
    r_val_min=h_r_min;
    if (h_ratio && h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: overriding ratio minimum with " << h_r_min << CNone << endl;
  }
  if (h_r_max!=0.) {
    r_val_max=h_r_max;
    if (h_ratio && h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: overriding ratio max with " << h_r_max << CNone << endl;
  }
  if (h_ratio && h_showinfo && fbins<0.1 && h_r_min==0. && h_r_max==0. ) {
    cout << "  " << CRed << "TrtBigPaperToolPlot: WARNING: showing only " << fbins << " of your ratio points!" << endl;
    cout << "                    You might be removing too many outliers (" << h_outliers << ")?" << CNone << endl;
  }

  if (h_ratio && h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: mean ratio and error: "
       << r_val_mean << " and " << r_err_mean << " (" << 100*r_err_mean/r_val_mean << "%)" << CNone << endl;

  h_0->SetStats(0);
  h_0->SetLineWidth(1);

  if (h_yellow) {
    h_0->SetMarkerSize(0.);
    h_0->SetMarkerStyle(0);
    h_0->SetFillColor(kYellow);
  } else {
    h_0->SetMarkerSize(1.);
    h_0->SetMarkerStyle(24);
    h_0->SetFillColor(kWhite);
  }

  h_0->GetXaxis()->SetTitle(s_x_title.c_str());
  h_0->GetYaxis()->SetTitle(s_y_title.c_str());

  if (h_ratio) {
    h_0->GetXaxis()->SetTitleSize(0.);
    h_0->GetXaxis()->SetLabelSize(0.);
    h_0->GetYaxis()->SetTitleSize(0.06);
    h_0->GetYaxis()->SetLabelSize(0.055);
    h_0->GetYaxis()->SetTitleOffset(1.3);
  } else {
    h_0->GetXaxis()->SetTitleSize(0.0425);
    h_0->GetXaxis()->SetLabelSize(0.042);
    h_0->GetYaxis()->SetTitleSize(0.045);
    h_0->GetYaxis()->SetLabelSize(0.04125);
    h_0->GetYaxis()->SetTitleOffset(1.73);
  }

  h_1->SetStats(0);
  h_1->SetLineColor(1);
  h_1->SetLineWidth(2);
  h_1->SetMarkerStyle(20);
  h_1->SetMarkerSize(1.);

  ratio_0_1->SetTitle("");
  ratio_0_1->SetStats(0);
  ratio_0_1->SetLineWidth(2);
  ratio_0_1->SetLineColor(1);
  ratio_0_1->SetMarkerStyle(24);
  ratio_0_1->SetMarkerSize(0.8);
  ratio_0_1->GetXaxis()->SetTitle(s_x_title.c_str());
  string r_title = "#frac{" + h_sample0 + "}{" + h_sample1 + "}";
  ratio_0_1->GetYaxis()->SetTitle(r_title.c_str());
  ratio_0_1->GetXaxis()->SetTitleOffset(1.0);
  ratio_0_1->GetXaxis()->SetTitleSize(0.17);
  ratio_0_1->GetYaxis()->SetTitleOffset(0.45);
  ratio_0_1->GetYaxis()->SetTitleSize(0.13);
  ratio_0_1->GetXaxis()->SetLabelSize(0.18);
  ratio_0_1->GetYaxis()->SetLabelSize(0.12);
  ratio_0_1->GetYaxis()->SetRangeUser(r_val_min-r_err_mean,r_val_max+r_err_mean); // all non-zero points are seen
  ratio_0_1->GetYaxis()->SetNdivisions(5);
  ratio_0_1->GetXaxis()->SetTickLength(0.12);
  ratio_0_1->GetYaxis()->SetTickLength(0.02);

  //canvas->Clear();

  TPad *pad1;
  if (h_ratio) {
    pad1 = new TPad("pad1","pad1", 0.0, 0.25, 1.0, 1.0);
    pad1->SetBottomMargin(0.022);
  } else {
    pad1 = new TPad("pad1","pad1", 0.0, 0.0, 1.0, 1.0);
  }

  pad1->Draw();
  pad1->cd();

  pad1->SetFillColor(0);
  pad1->SetBorderMode(0);
  pad1->SetBorderSize(2);
  pad1->SetTickx(1);
  pad1->SetTicky(1);
  pad1->SetLeftMargin(0.16);
  pad1->SetRightMargin(0.05);
  pad1->SetTopMargin(0.05);
  pad1->SetFrameBorderMode(0);

  if (h_logy) pad1->SetLogy();

  if (h_yellow) {
    h_0->Draw("hist");
  } else {
    if (h_errorbars) h_0->Draw("p");
    else             h_0->Draw("hist p");
  }
  if (!h_one) {
    if (h_errorbars) h_1->Draw("p,same");
    else             h_1->Draw("hist p,same");
  }
  TLatex *latex = new TLatex();
  latex->SetNDC();
  latex->SetTextSize(0.06);

  if (!h_statbox) {
    if (h_ratio) {
      latex->SetTextFont(72);
      latex->SetTextSize(0.06);
      latex->DrawLatex(0.20,0.85,"ATLAS");
      latex->SetTextFont(42);
      latex->DrawLatex(0.36,0.85,h_status.c_str());
    } else {
      latex->SetTextFont(72);
      latex->SetTextSize(0.045);
      latex->DrawLatex(0.20,0.87,"ATLAS");
      latex->SetTextFont(42);
      latex->DrawLatex(0.36,0.87,h_status.c_str());
    }
  }

  // up to 10 lines of latex (0-9)
  latex->SetTextSize(0.050);
  latex->SetTextFont(42);
  for (int i=0; i<10; i++) latex->DrawLatex(text[i].x,text[i].y,(text[i].s).c_str());
  // statbox (10-19),(20-29)
  latex->SetTextSize(0.020);
  latex->SetTextFont(82);
  for (int i=10; i<30; i++) latex->DrawLatex(text[i].x,text[i].y,(text[i].s).c_str());


  TLegend *legend;
  if (h_ratio) {
    legend = new TLegend(0.65,0.76,0.92,0.91);
  } else {
    legend = new TLegend(0.65,0.80,0.92,0.91);
  }

  legend->SetBorderSize(0);

  if (h_yellow) {
    h_0->SetLineWidth(1);
    legend->AddEntry(h_0,h_sample0.c_str(),"f");
  } else {
    legend->AddEntry(h_0,h_sample0.c_str(),"p");
  }

  if (h_one) {if (h_showinfo) cout << "  " << CBlue << "TrtBigPaperToolPlot: displaying only one histogram" << CNone << endl;}
  else legend->AddEntry(h_1,h_sample1.c_str(),"p");

  legend->Draw();

  //
  // ratio plot
  //
  if (h_ratio) {
    canvas->cd();
    TPad *pad2 = new TPad("pad2","pad2", 0.0, 0.0, 1.0, 0.25);
    pad2->SetTopMargin(0.);
    pad2->SetBottomMargin(0.4);
    pad2->Draw();
    pad2->cd();
    pad2->SetFillColor(0);
    pad2->SetBorderMode(0);
    pad2->SetBorderSize(2);
    pad2->SetTickx(1);
    pad2->SetTicky(1);
    pad2->SetLeftMargin(0.16);
    pad2->SetRightMargin(0.05);
    pad2->SetFrameBorderMode(0);

    if (h_errorbars) ratio_0_1->Draw("p"); else ratio_0_1->Draw("p hist");

    TLine *line1 = new TLine(h_xmin,r_val_mean,h_xmax,r_val_mean);
    line1->SetLineColor(kBlack);
    line1->SetLineWidth(2);
    line1->SetLineStyle(7);
    line1->Draw("same");

    bool inPad = r_val_min-r_err_mean<1. && r_val_max+r_err_mean>1.;
    if (inPad) {
      TLine *line2 = new TLine(h_xmin,1.,h_xmax,1.);
      line2->SetLineColor(kBlack);
      line2->SetLineWidth(2);
      line2->SetLineStyle(1);
      line2->Draw("same");
    }
  } // end ratio plot

  //string pdf = canvas_name+".pdf";
  string png = canvas_name+".png";
  //string jpg = canvas_name+".jpg";
  //string mac = canvas_name+".C";
  //canvas->Print(pdf.c_str());
  canvas->Print(png.c_str());
  //canvas->Print(jpg.c_str());
  //canvas->Print(mac.c_str());

}
